﻿Option Explicit On

' Delcom Products - VB.Net Example
' Sample program finds, opens, read, writes and closes the USB Device.
' Works with all Delcom USB HID devices. VID(Vendor ID) = 0x0FC5, PID(Product ID) = 0xB080

' --- This program requires the following files, add these to your project ---
' DeviceManagement.vb
' DeviceManagementDeclarations.vb
' FileIODeclarations.vb
' HidDeclarations.vb
' Hid.vb

Imports System.Globalization
Imports System.Runtime.InteropServices
Imports Microsoft.Win32.SafeHandles
Imports HID_VB_NET_APP.DeviceManagement
Imports HID_VB_NET_APP.FileIO
Imports HID_VB_NET_APP.Hid


Friend Class Delcom

    Const MODULE_NAME As String = "Delcom"



    ' ----------------- Class Data Structures ----------------------

    ' ----------------- Class Data Structures ----------------------
    ' InfoStruct - A structure to hold the returned device info, Cmd 10
    Structure InfoStruct
        Public Serial As Int32
        Public Version As Byte
        Public Day As Byte
        Public Month As Byte
        Public Year As Byte
    End Structure
    ' InfoExtStruct - A structure to hold the returned device info, Cmd ???
    Structure InfoExtStruct
        Public Family As Int16
        Public Security As Int16
        Public Version As Byte
        Public Day As Byte
        Public Month As Byte
        Public Year As Byte
        Public Serial As Int32
        Public Spare As Int32
    End Structure
    ' RxPacketStruct - A structure to send and received data packets from the USB device
    Structure TxPacketStruct
        Public MajorCmd As Byte
        Public MinorCmd As Byte
        Public DataLSB As Byte
        Public DataMSB As Byte
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=4)> Dim HidData() As Byte
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=8)> Dim ExtData() As Byte
    End Structure
    ' RxPacketStruct - A structure to send and received data packets from the USB device
    Structure RxPacketStruct
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=16)> Dim Data() As Byte
    End Structure


    ' ----------------- Class Variables ----------------------------
    Public DeviceName As String
    Public Info As InfoStruct

    Private Detected As Boolean
    Private Openned As Boolean
    Private hidHandle As SafeFileHandle
    Private hidUsage As String
    Public MyDeviceManagement As New DeviceManagement()
    Private deviceNotificationHandle As IntPtr
    Private MyHid As New Hid()
    Private usbHandle As SafeFileHandle
    Private TxPacket As TxPacketStruct
    Private RxPacket(16) As Byte





    ' ----------------- Class Functions ----------------------
    ''' <summary>
    ''' New() - Constructor, initialize class
    ''' </summary>
    Public Sub New()
        Detected = False
        Openned = False
    End Sub


    ''' <summary>
    ''' IsOpen() - Returns true is device is openned, else false
    ''' </summary>
    Function IsOpen() As Boolean
        If Openned Then
            Return True
        Else
            Return False
        End If
    End Function


    ''' <summary>
    ''' OpenDevice() - Opens the device. If devicename not passed as argument, then will use local devicename.
    ''' </summary>
    '''         
    ''' <returns>
    '''  True if the device is open
    ''' </returns>
    Public Function OpenDevice(Name As String) As Boolean
        If Name.Length = 0 Then
            Name = DeviceName
        Else
            DeviceName = Name
        End If
        usbHandle = CreateFile(Name, 0, FILE_SHARE_READ Or FILE_SHARE_WRITE, _
                  IntPtr.Zero, OPEN_EXISTING, 0, 0)

        If Not (hidHandle.IsInvalid) Then
            Openned = True
            Return True
        Else
            Return False
        End If
    End Function


    ''' <summary>
    ''' CloseDevice() - Close the device. 
    ''' </summary>
    '''         
    ''' <returns>
    '''  True if the device is closed
    ''' </returns>
    Public Function CloseDevice() As Boolean
        Try
            If Not usbHandle Is Nothing Then
                If Not usbHandle.IsInvalid Then
                    usbHandle.Close()
                End If
            End If
            If Detected Then
                MyDeviceManagement.StopReceivingDeviceNotifications(deviceNotificationHandle)
            End If
            Detected = False
            Openned = False
            Return True
        Catch ex As Exception
            'DisplayException(Me.Name, ex)
            Throw
        End Try
    End Function


    ''' <summary>
    ''' FindDevice() - Scans for the Delcom USB device by its Vendor ID and Product ID.
    ''' </summary>
    '''         
    ''' <returns>
    '''  True if the device is detected, False if not detected.
    ''' </returns>
    Public Function FindDevice() As Boolean

        Dim deviceFound As Boolean
        Dim devicePathName(512) As String
        Dim hidGuid As System.Guid
        Dim memberIndex As Int32
        Dim myProductID As Int32
        Dim myVendorID As Int32
        Dim success As Boolean

        Try
            Detected = False

            myVendorID = &HFC5
            myProductID = &HB080

            HidD_GetHidGuid(hidGuid)    ' Gets and init the HID Class

            ' Fill an array with the device path names of all attached HIDs.
            deviceFound = MyDeviceManagement.FindDeviceFromGuid(hidGuid, devicePathName)

            ' Device(s) found, scan each one for our device
            If deviceFound Then
                memberIndex = 0
                Do
                    ' ***
                    ' API function:
                    ' CreateFile

                    ' Purpose:
                    ' Retrieves a handle to a device.

                    ' Accepts:
                    ' A device path name returned by SetupDiGetDeviceInterfaceDetail
                    ' The type of access requested (read/write).
                    ' FILE_SHARE attributes to allow other processes to access the device while this handle is open.
                    ' A Security structure or IntPtr.Zero. 
                    ' A creation disposition value. Use OPEN_EXISTING for devices.
                    ' Flags and attributes for files. Not used for devices.
                    ' Handle to a template file. Not used.

                    ' Returns: a handle without read or write access.
                    ' This enables obtaining information about all HIDs, even system
                    ' keyboards and mice. 
                    ' Separate handles are used for reading and writing.
                    ' ***

                    hidHandle = CreateFile(devicePathName(memberIndex), 0, FILE_SHARE_READ Or FILE_SHARE_WRITE, _
                        IntPtr.Zero, OPEN_EXISTING, 0, 0)

                    If Not (hidHandle.IsInvalid) Then ' If valid return handle
                        ' Set the Size property of DeviceAttributes to the number of bytes in the structure.
                        MyHid.DeviceAttributes.Size = Marshal.SizeOf(MyHid.DeviceAttributes)
                        ' Get the artibute structure
                        success = HidD_GetAttributes(hidHandle, MyHid.DeviceAttributes)

                        If success Then ' Find out if the device matches the one we're looking for.
                            If (MyHid.DeviceAttributes.VendorID = myVendorID) And _
                                (MyHid.DeviceAttributes.ProductID = myProductID) Then  ' DEVICE MATCH!
                                Detected = True     ' mark device found
                                DeviceName = devicePathName(memberIndex) ' save the decivename
                            Else    ' It's not a match, so close the handle.
                                Detected = False
                                hidHandle.Close()
                            End If
                        Else    ' error is read atributes
                            Detected = False
                            hidHandle.Close()
                        End If

                    End If

                    ' Keep looking until we find the device or there are no devices left to examine.
                    memberIndex = memberIndex + 1
                Loop Until (Detected Or (memberIndex = devicePathName.Length))
            End If

            If Detected Then    ' The device was detected.

                'Optional - Register to receive notifications if the device is removed or attached.
                'comment out if don't what these
                success = MyDeviceManagement.RegisterForDeviceNotifications _
                    (DeviceName, Form1.Handle, hidGuid, deviceNotificationHandle)

            End If

            Return Detected

        Catch ex As Exception
            DisplayException(Form1.Name, ex)
            Throw
        End Try
    End Function



    ''' <summary>
    ''' WriteDevice(TxPacket) - Sends a write packet to the USB device
    ''' See the USB HID data sheet for command info
    ''' </summary>
    Function WriteDevice(TxPacket As TxPacketStruct) As Boolean
        Dim success As Boolean
        Dim ByteArray() As Byte
        Dim Ptr As IntPtr = Marshal.AllocHGlobal(Marshal.SizeOf(TxPacket))
        ReDim ByteArray(Marshal.SizeOf(TxPacket) - 1)
        Marshal.StructureToPtr(TxPacket, Ptr, False)
        Marshal.Copy(Ptr, ByteArray, 0, Marshal.SizeOf(TxPacket))
        success = HidD_SetFeature(usbHandle, ByteArray, 16)
        Marshal.FreeHGlobal(Ptr)
        Return success
    End Function

    ''' <summary>
    ''' ReadDevice(Cmd, RxPacket) - Sends a read packet to the USB device
    ''' See the USB HID data sheet for command info
    ''' </summary>
    Function ReadDevice(Cmd As Byte, ByRef rxData() As Byte) As Integer
        Dim success As Boolean
        rxData(0) = Cmd
        success = HidD_GetFeature(usbHandle, rxData, 16)
        Return success
    End Function




    ''' <summary>
    ''' GetDeviceInfo() - Gets the device info
    ''' returns device version number on success, else zero
    ''' </summary>
    Function GetDeviceInfo() As Integer
        Dim success As Boolean
        success = ReadDevice(10, RxPacket)
        If success Then
            Info = Copy2Struct(RxPacket, Info.GetType)
            Return Info.Version
        Else
            Return 0
        End If
    End Function

    ''' <summary>
    ''' ReadPorts() - Reads the ports
    ''' returns true on sucess
    ''' </summary>
    Function ReadPorts(ByRef Port0 As Byte, ByRef Port1 As Byte) As Boolean
        Dim success As Boolean
        success = ReadDevice(100, RxPacket)
        If success Then
            Port0 = RxPacket(0)
            Port1 = RxPacket(1)
        End If
        Return success
    End Function


    ''' <summary>
    ''' WritePort(Port#, Value) - Write Port
    ''' </summary>    
    Function WritePort(Port As Byte, Value As Byte) As Boolean
        TxPacket.MajorCmd = 102
        TxPacket.DataLSB = Value
        If Port = 0 Then
            TxPacket.MinorCmd = 1
        Else
            If Port = 1 Then
                TxPacket.MinorCmd = 2
            Else
                Return False
            End If
        End If

        Return WriteDevice(TxPacket)
    End Function

    ''' <summary>
    ''' LEDCtrl(LED#, Mode) - LED Ctrl
    ''' </summary>    
    Function LEDCtrl(LED As Byte, Mode As Byte) As Boolean
        Dim success As Boolean
        success = True

        ' 1) - Always turn flash mode off first
        TxPacket.MajorCmd = 102
        TxPacket.MinorCmd = 20  ' flash cmd
        TxPacket.DataLSB = (&H1 << LED) '  off
        TxPacket.DataMSB = &H0
        WriteDevice(TxPacket)

        ' 2) - Always turn all leds off first
        TxPacket.MajorCmd = 102
        TxPacket.MinorCmd = 12  ' port1 wr cmd
        TxPacket.DataLSB = &H0
        TxPacket.DataMSB = (&H1 << LED) ' led off
        WriteDevice(TxPacket)

        ' 3) - if mode = 1(on) then turn on the led
        If Mode = 1 Or Mode = 2 Then
            TxPacket.MajorCmd = 102
            TxPacket.MinorCmd = 12  ' port1 wr cmd
            TxPacket.DataLSB = (&H1 << LED) ' led on
            TxPacket.DataMSB = &H0
            WriteDevice(TxPacket)
        End If

        If Mode = 2 Then
            ' 4) - if mode = 2(flash) then flash the led
            TxPacket.MajorCmd = 102
            TxPacket.MinorCmd = 20  ' flash cmd
            TxPacket.DataLSB = &H1
            TxPacket.DataMSB = (&H1 << LED) ' flash on
            WriteDevice(TxPacket)
        End If


        Return success
    End Function

    ''' <summary>
    ''' LEDPwr(LED#, value) - LED power
    ''' </summary>    
    Function LEDPwr(LED As Byte, value As Byte) As Boolean
        TxPacket.MajorCmd = 102
        TxPacket.MinorCmd = 34  ' power cmd
        TxPacket.DataLSB = LED '  
        TxPacket.DataMSB = value
        Return (WriteDevice(TxPacket))
    End Function

    ' Function to copy byte array into a structure
    Private Function Copy2Struct(ByVal Buff() As Byte, _
               ByVal MyType As System.Type) As Object
        Dim MyGC As GCHandle = GCHandle.Alloc(Buff, GCHandleType.Pinned)
        'Marshals data from an unmanaged block of memory 
        'to a newly allocated managed object of the specified type.
        Dim Obj As Object = _
            Marshal.PtrToStructure(MyGC.AddrOfPinnedObject, MyType)
        Return Obj
        'Free GChandle to avoid memory leaks
        MyGC.Free()
    End Function


End Class
