﻿' Delcom Products - VB.Net Example
' Sample program finds, opens, read, writes and closes the USB Device.
' Works with all Delcom USB HID devices. VID(Vendor ID) = 0x0FC5, PID(Product ID) = 0xB080

' Form1.vb  - Form functions
' Delcom.vb - Delcom funcitons

' --- This program requires the following files, add them to your project ---
' DeviceManagement.vb
' DeviceManagementDeclarations.vb
' FileIODeclarations.vb
' HidDeclarations.vb
' Hid.vb
' Delcom.vb

Imports HID_VB_NET_APP.DeviceManagement



Public Class Form1

    Dim myDelcom As HID_VB_NET_APP.Delcom = New Delcom()


    ' Finds/opens oe closes the device.
    Private Sub ButtonOpen_Click(sender As Object, e As EventArgs) Handles ButtonOpen.Click
        Try
            If myDelcom.IsOpen() Then   ' if device is already open close it
                myDelcom.CloseDevice()
                ButtonOpen.Text = "Open"
                Label_Status.Text = "Status: Device closed"
                Label_DeviceName.Text = "DeviceName:"
                Label_DeviceInfo.Text = "DeviceInfo:"

            Else                        ' else find the device and open it

                If (myDelcom.FindDevice()) Then     ' find the device
                    Label_Status.Text = "Status: Found Device"
                    Label_DeviceName.Text = "DeviceName: " + myDelcom.DeviceName

                    If (myDelcom.OpenDevice("")) Then   ' open the device
                        Label_Status.Text = "Status: Device Openned!"
                        ButtonOpen.Text = "Close"
                        myDelcom.GetDeviceInfo()
                        Label_DeviceInfo.Text = "DeviceInfo: Firmware Version " + CStr(myDelcom.Info.Version) _
                        + "  " + CStr(myDelcom.Info.Month) + "/" + CStr(myDelcom.Info.Day) + "/" + CStr(2000 + myDelcom.Info.Year)

                    Else
                        Label_Status.Text = "Status: Device detected but failed to open device"
                        Label_DeviceName.Text = "DeviceName:"
                        Label_DeviceInfo.Text = "DeviceInfo:"

                    End If

                Else
                    Label_Status.Text = "Found NOT Device"
                    Label_DeviceName.Text = ""
                    Label_DeviceInfo.Text = ""
                End If
            End If
        Catch ex As Exception
            DisplayException(Me.Name, ex)
            'Throw
        End Try
    End Sub


    ' Closes the window.
    Private Sub Button_Close(sender As Object, e As EventArgs) Handles ButtonQuit.Click
        myDelcom.CloseDevice()
        Close()
    End Sub


    Private Sub Button_ReadPorts_Click(sender As Object, e As EventArgs) Handles Button_ReadPorts.Click
        Try
            Dim P0, P1 As Byte
            If Not (IsOpen()) Then
                Return
            End If

            myDelcom.ReadPorts(P0, P1)
            TextBox_RdP0.Text = P0
            TextBox_RdP1.Text = P1
        Catch ex As Exception
            DisplayException(Me.Name, ex)
            'Throw
        End Try
    End Sub

    Private Sub Button_LED0_OFF_Click(sender As Object, e As EventArgs) Handles Button_LED_OFF.Click
        Try
            If Not (IsOpen()) Then
                Return
            End If
            myDelcom.LEDCtrl(ComboBox_LED.SelectedIndex, 0)
        Catch ex As Exception
            DisplayException(Me.Name, ex)
            'Throw
        End Try
    End Sub

    Private Sub Button_LED_ON_Click(sender As Object, e As EventArgs) Handles Button_LED_ON.Click
        Try
            If Not (IsOpen()) Then
                Return
            End If
            myDelcom.LEDCtrl(ComboBox_LED.SelectedIndex, 1)
        Catch ex As Exception
            DisplayException(Me.Name, ex)
            'Throw
        End Try
    End Sub

    Private Sub Button_LED_FLASH_Click(sender As Object, e As EventArgs) Handles Button_LED_FLASH.Click
        Try
            If Not (IsOpen()) Then
                Return
            End If
            myDelcom.LEDCtrl(ComboBox_LED.SelectedIndex, 2)
        Catch ex As Exception
            DisplayException(Me.Name, ex)
            'Throw
        End Try
    End Sub

 
    Private Sub Button_WriteP0_Click(sender As Object, e As EventArgs) Handles Button_WriteP0.Click
        Try
            If Not (IsOpen()) Then
                Return
            End If
            myDelcom.WritePort(0, CByte(TextBox_WrP0.Text))
        Catch ex As Exception
            DisplayException(Me.Name, ex)
            'Throw
        End Try
    End Sub

    Private Sub Button_WriteP1_Click(sender As Object, e As EventArgs) Handles Button_WriteP1.Click
        Try
            If Not (IsOpen()) Then
                Return
            End If
            myDelcom.WritePort(1, CByte(TextBox_WrP1.Text))
        Catch ex As Exception
            DisplayException(Me.Name, ex)
            'Throw
        End Try
    End Sub

    Private Sub Button_LED_POWER_Click(sender As Object, e As EventArgs) Handles Button_LED_POWER.Click
        Try
            If Not (IsOpen()) Then
                Return
            End If
            myDelcom.LEDPwr(ComboBox_LED.SelectedIndex, CByte(TextBox_Power.Text))
        Catch ex As Exception
            DisplayException(Me.Name, ex)
            'Throw
        End Try
    End Sub


    Private Function IsOpen() As Boolean
        If Not (myDelcom.IsOpen()) Then
            MessageBox.Show("Device not openned. Open device first!", "Warning", MessageBoxButtons.OK)
            Return False
        Else
            Return True
        End If
    End Function


    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        TextBox_WrP0.Text = "255"
        TextBox_WrP1.Text = "255"
        TextBox_Power.Text = "100"
        ComboBox_LED.SelectedIndex = 0
    End Sub

    ''' <summary>
    ''' Provides a central mechanism for exception handling.
    ''' Displays a message box that describes the exception.
    ''' </summary>
    ''' 
    ''' <param name="moduleName"> the module where the exception occurred. </param>
    ''' <param name="e"> the exception </param>

    Shared Sub DisplayException(ByVal moduleName As String, ByVal e As Exception)

        Dim message As String
        Dim caption As String

        ' Create an error message.

        message = "Exception: " & e.Message & ControlChars.CrLf & _
        "Module: " & moduleName & ControlChars.CrLf & _
        "Method: " & e.TargetSite.Name

        caption = "Unexpected Exception"

        MessageBox.Show(message, caption, MessageBoxButtons.OK)
        Debug.Write(message)

    End Sub

    ''' <summary>
    '''  Overrides WndProc to enable checking for and handling WM_DEVICECHANGE messages.
    ''' </summary>
    ''' 
    ''' <param name="m"> a Windows Message </param>

    Protected Overrides Sub WndProc(ByRef m As Message)

        Try
            ' The OnDeviceChange routine processes WM_DEVICECHANGE messages.
            If m.Msg = WM_DEVICECHANGE Then
                OnDeviceChange(m)
            End If

            MyBase.WndProc(m)   ' when done pass ctrl to the base class

        Catch ex As Exception
            DisplayException(Me.Name, ex)
            Throw
        End Try

    End Sub


    ''' <summary>
    ''' Called when a WM_DEVICECHANGE message has arrived,
    ''' indicating that a device has been attached or removed.
    ''' </summary>
    ''' 
    ''' <param name="m"> a message with information about the device </param>

    Friend Sub OnDeviceChange(ByVal m As Message)

        Debug.WriteLine("WM_DEVICECHANGE")

        Try
            If (m.WParam.ToInt32 = DBT_DEVICEARRIVAL) Then
                ' If WParam contains DBT_DEVICEARRIVAL, a device has been attached.
                ' Find out if it's the device we're communicating with.
                If myDelcom.MyDeviceManagement.DeviceNameMatch(m, myDelcom.DeviceName) Then
                    MessageBox.Show("Device has been attached.", "Message", MessageBoxButtons.OK)
                End If

            ElseIf (m.WParam.ToInt32 = DBT_DEVICEREMOVECOMPLETE) Then
                ' If WParam contains DBT_DEVICEREMOVAL, a device has been removed.


                ' Find out if it's the device we're communicating with.

                If myDelcom.MyDeviceManagement.DeviceNameMatch(m, myDelcom.DeviceName) Then
                    MessageBox.Show("Device has been removed.", "Message", MessageBoxButtons.OK)
                    myDelcom.CloseDevice()
                    ButtonOpen.Text = "Open"
                End If
            End If



        Catch ex As Exception
            DisplayException(Me.Name, ex)
            Throw
        End Try
    End Sub

End Class